/* 
 * THC-Shagg - A modular application to bruteforce check digit algorithms
 * (c) 2001, 2003 Plasmoid <plasmoid@thc.org>, The Hacker's Choice
 * $Id: State.java,v 1.10 2003/07/18 10:08:05 plasmoid Exp $ 
 */

package org.thc.shagg;
import org.thc.shagg.algorithms.*;

import java.util.Vector;
import java.util.Map;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;
import java.io.*;

/**
 * This class represents a state during the analysis of a set of serial a
 * number. A state holds information about the used amount of serial numbers,
 * algorithm, weights, and partition that matched a serial number. During
 * further progress the hits of a state are increased if he matches another
 * serial number.
 * <p>
 * In order to store the states in an efficient data structure this class
 * implements the interface comparable and the states may be stored in a
 * tree map or any other sorted map that sorts on comparable objects.
 * 
 * @version $Id: State.java,v 1.10 2003/07/18 10:08:05 plasmoid Exp $
 * @author Plasmoid (plasmoid@thc.org),
 *                  <a href="http://www.thc.org">THC</a>
 */
public class State implements Comparable,Serializable {
   private int[] weights, partition;
   private int algorithm;
   private int amount;
   private int hits;
   private String id;

   /** 
    * Construct a state.
    * @param w Weights that matched.
    * @param a Number of algorithm that matched.
    * @param p Partition that matched.
    * @param l Length of serial number
    */
   public State(int[] w, int a, int[] p, int l) {
     
      weights=w;
      algorithm=a;
      partition=p;
      hits=1;
      amount=l;
      
      id=generateUniqueId();
   }
   
   /**
    * Generates a unique id.
    * In order to perform sorting on a given set it is necessary for the 
    * elements of the set have a total ordering. This unique id helps 
    * implementing a compareTo() method. 
    * @see #compareTo
    * @return id represented by a string. 
    */
   private String generateUniqueId() {
      String id;
      
      id=algorithm+"-a";
      id+=amount+"-n";
      for(int i=0;i<partition.length;i++)
         id+=partition[i]+"-";
      id+="p";

      for(int i=0;weights!=null && i<weights.length;i++)
         id+=weights[i]+"-";
      id+="w";   
   
      return id;
   }
   
   /**
    * Hit! Increase hit counter. 
    */
   public void hit() {
      hits++;
   }

   /** 
    * Get amount of serial numbers that were used generarting this state.
    * @return amount
    */
   public int getAmout() {
      return amount;
   }

   /** 
    * Get hit counter.
    * @return Hits.
    */
   public int getHits() {
      return hits;
   }

   /** 
    * Get the unique id of this state.
    * @return Id.
    */
   public String getId() {
      return id;
   }

   /**
    * Get weights of this state.
    * @return Weights.
    */
   public int[] getWeights() {
      return weights;
   }
   
   /**
    * Get algorithm of this state.
    * @return Number of algorithm
    */
   public int getAlgorithm() {
      return algorithm;
   }
   
   /**
    * Get partition of this state.
    * @return Partition.
    */
   public int[] getPartition() {
      return partition;
   }
   
   /**
    * Get length of serial number.
    * @return Length
    */
   public int getLength() {
      int len=0;
      
      for(int i=0;i<partition.length;i++)
         len+=partition[i];
         
      return len;
   } 
   
   /**
    * Check if two states are equal.
    * This method is based on the equals() method of the String class.
    * @see java.lang.String#equals
    * @param s State to compare with this state.
    * @return true or false.
    */
   public boolean equals(State s) {
      return id.equals(s.getId());
   }
   
   /** 
    * Compares two states.
    * This method is based on the compareTo() method of the String class.
    * @see java.lang.String#compareTo
    * @param s State to compare with this state.
    * @return -1 this state is "smaller", 0 states are equal, 1 this state is
    *         "bigger".
    */
   public int compareTo(Object s) {
      return id.compareTo(((State)s).getId());
   }

   /**
    * Find a state in a map and give it a hit.  
    * The method performs a find operation on the given map, if the state is
    * not found it is added to the map, otherwise it receives another hit.
    * This function is very time-consuming, I recommand using data structure
    * that perform the find and the addition in O(log(n)), e.g. a red-black
    * tree as used in the Java util implementation.
    * @see java.util.Map
    * @see java.util.SortedMap
    * @param t Map
    */
   public void hit(Map t) {
      boolean found=false;
      State s=null;
      
      s=(State)t.get(id);
      if(s==null) {
         t.put(id,this);
      } else {
         s.hit();
      }
   }

   /**
    * Clear map from states that don't have enough hits. 
    * This function reads the entries from the given map and checks if 
    * each entry has more than limit hits, if not the state is removed. 
    * Runtime O(n), this is expensive!
    * @param t Map.
    * @param limit Limit for hits.
    * @return Vector of all states that have enough hits.
    */ 
   public static Vector clear(Map t, int limit) {
      State s=null;
      Vector v=new Vector(t.values());
      
      for(int i=0;i<v.size();i++) {
         s=(State)v.get(i);
         if(s.getHits()<limit)
            v.remove(i--);
      }   
      
      return v;
   }
   
   /**
    * Print state.
    * The name of the algorithm and the length of the serial number analysed
    * are obtained through the parameters.
    * @see org.thc.shagg.algorithms.CheckDigitAlgorithm
    * @param algorithms Array of check digit algorithms
    * @param num Length of the serial number. 
    */
   public void print(CheckDigitAlgorithm[] algorithms) {

      System.out.println("Algorithm: "+algorithms[algorithm].getName());
      System.out.print("Partition: ");

      for(int i=0;i<partition[0];i++) 
         System.out.print("*");
         
      for(int i=0;i<partition[1];i++) {
         if(algorithms[algorithm].getCheckDigitPosition(partition[1])==i)
            System.out.print("C");      
         else
            System.out.print("X");   
      }
      
      for(int i=0;i<partition[2];i++) 
         System.out.print("*");      
         
      System.out.println();

      if(algorithms[algorithm].usesWeights()) {
         System.out.print("  Weights: ");
         
         for(int j=0;j<weights.length;j++)
            System.out.print(weights[j]);
            
         for(int j=0;j<weights.length;j++)   
            System.out.print(".");
            
         System.out.println();
      }

      System.out.println("     Hits: "+
                         Math.round((hits/(double)amount)*100)+"%\n");
   }   
   
   /**
    * Load a state from a file.
    * The file is compressed using the gzip compression provided by Java.
    * @param filename Name of file.
    * @return The loaded state.
    * @throws IOException if anything goes wrong
    * @throws ClassNotFoundException if the loaded file is not a correct class
    */
   public static State load(String filename) throws IOException, 
                                             ClassNotFoundException {
      State s;
      GZIPInputStream zip = new GZIPInputStream(new     
                            FileInputStream(filename)); 
      ObjectInputStream in = new ObjectInputStream(zip);
      s = (State)in.readObject();
      in.close();
      return s;
   }

   /**
    * Save a state to a file.
    * The file is compressed using the gzip compression provided by Java.
    * @param filename Name of file.
    * @throws IOException if anything goes wrong
    */
   public void save(String filename) throws IOException {
      GZIPOutputStream zip = new GZIPOutputStream(new   
                             FileOutputStream(filename));
      ObjectOutputStream out = new ObjectOutputStream(zip);
      out.writeObject(this);
      out.close();
   }

   /**
    * Load vector of states from a file.
    * The file is compressed using the gzip compression provided by Java.    
    * @param filename Name of file.
    * @return The loaded vector.
    * @throws IOException if anything goes wrong.
    * @throws ClassNotFoundException if the loaded file is not a correct class
    */
   public static Vector loadVector(String filename) throws IOException, 
                                                    ClassNotFoundException {
      Vector v;
      GZIPInputStream zip = new GZIPInputStream(new     
                            FileInputStream(filename)); 
      ObjectInputStream in = new ObjectInputStream(zip);
      v = (Vector)in.readObject();
      in.close();
      return v;
   }

   /**
    * Save a vector of states to a file.
    * The file is compressed using the gzip compression provided by Java.
    * @param filename Name of file.
    * @param v Vector of states.
    * @throws IOException if anything goes wrong.
    */
   public static void saveVector(String filename, Vector v) throws 
                                                            IOException {
      GZIPOutputStream zip = new GZIPOutputStream(new   
                             FileOutputStream(filename));
      ObjectOutputStream out = new ObjectOutputStream(zip);
      out.writeObject(v);
      out.close();
   }
   
   /** 
    * Checks a serial number against the data from this state.
    * @param number Serial number
    * @param a Check Digit algorithm to use
    * @return true or false
    */
   public boolean check(int[] number, CheckDigitAlgorithm[] a) {    
      int[] subNumber;

      subNumber = new int[partition[1]];
      for(int j=0;j<subNumber.length;j++)
         subNumber[j]=number[partition[0]+j];
         
      return a[algorithm].check(subNumber, weights);
   }

}
