/* 
 * THC-Shagg - A modular application to bruteforce check digit algorithms
 * (c) 2001, 2003 Plasmoid <plasmoid@thc.org>, The Hacker's Choice
 * $Id: Modulus10Algorithm.java,v 1.7 2003/07/18 10:08:05 plasmoid Exp $ 
 */
package org.thc.shagg.algorithms;

/**
 * Abstract class for the Modulus 10 Algorithm. The Modulus 10 Algorithm
 * is the most used check digit algorithm amoung the simple check digit
 * algorithms. It is even used in critial areas such as credit card and
 * calling card verification and has caused much fraud in the 90s.
 * <p>
 * In order to cover the variants of the Modulus 10 Algorithm, this 
 * abstract class was designed as a framework for all variant implementations. 
 *
 * @version $Id: Modulus10Algorithm.java,v 1.7 2003/07/18 10:08:05 plasmoid Exp $
 * @author Plasmoid (plasmoid@thc.org),
 *                  <a href="http://www.thc.org">THC</a>
 */
public abstract class Modulus10Algorithm implements CheckDigitAlgorithm 
{
   protected String type;

   /**
    * Get the name of the implemented check digit algorithm, Modulus 10.
    * The type field is implemented by the indiviual class extending this
    * abstract class.
    * @return algorithm name.
    */
   public String getName() {
      return "Modulus10 Algorithm ("+type+")";
   }

   /**
    * Get the symbol of the implemented check digit algorithm. In this
    * case "10" for Modulus 10.
    */
   public String getSymbol() {
      return "10"+type;
   }

   /**
    * Checks if the supplied sequence matches the Modulus 10 Algorithm.
    * Modulus uses the provided weights.
    * @param sequence Integer sequence representing the digits to check,
    *                 not necessary between 0 and 9.
    * @param weights  Integer sequence of the weights to use, may be of
    *                 any length.
    * @return wether sequences matches the algorithm.
    */
   public boolean check(int[] sequence, int[] weights) {
      long result=0;

      for(int i=0; i<sequence.length-1; i++) 
         result+=mod10Add(sequence,weights,i);

      return (result%10)==mod10checkDigit(sequence);
   }
   
   /**
    * Add method for the Modulus 10 Algorithm. The different versions 
    * of the Modulus 10 Algorithm differ in the way the digits are 
    * added.
    * @param sequence Integer sequence representing the digits to check,
    *                 not necessary between 0 and 9.
    * @param weights  Integer sequence of the weights to use, may be of
    *                 any length.
    * @param index    Current index
    * @return Sum at index.
    */
   public abstract int mod10Add(int[] sequence, int[] weights, int index);

   /**
    * Check method for the Modulus 10 Algorithm. The different versions 
    * of the Modulus 10 Algorithm differ in the way the check digit
    * is interpreted.
    * @param sequence Integer sequence representing the digits to check,
    *                 not necessary between 0 and 9.
    * @return The check digit.
    */
   public abstract int mod10checkDigit(int[] sequence);
   
   /**
    * The Modulus 10 Algorithm uses weights.
    * @return true
    */
   public boolean usesWeights() {
      return true;
   }

   /**
    * Return the check digit position inside the sequence. Modulus 10 uses
    * the last digit.
    * @param sequence Length of the integer sequence representing the digits
    *                 to check, not necessary between 0 and 9.
    * @return Position of the check digit.
    */
   public int getCheckDigitPosition(int sequence) {
      return sequence-1;
   }
}
