/* 
 * THC-Shagg - A modular application to bruteforce check digit algorithms
 * (c) 2001, 2003 Plasmoid <plasmoid@thc.org>, The Hacker's Choice
 * $Id: Modulus11Algorithm.java,v 1.7 2003/07/18 10:08:05 plasmoid Exp $ 
 */
package org.thc.shagg.algorithms;

/**
 * Implementation of the Modulus 11 Algorithm commonly used in the ISBN and
 * EAN numbers. The digits are multiplied with the reversed index and summed
 * up. 11 subtracted from the modulus 11 of the sum S is the final check
 * digit C. If the modulus 11 operation results 0, the check digit needs to
 * be zero instead of 11.
 * <ul><tt>
 *   1 2 3 4 ... 9   C <br>
 *   * * * *     *     <br>
 *   9 8 7 6 ... 1     <br>
 *    + + +  ...+ -> S <br>
 * 11 - (S mod 11) = C <br>
 * <tt></ul>
 *
 * @version $Id: Modulus11Algorithm.java,v 1.7 2003/07/18 10:08:05 plasmoid Exp $
 * @author Plasmoid (plasmoid@thc.org),
 *                  <a href="http://www.thc.org">THC</a>
 */
public class Modulus11Algorithm implements CheckDigitAlgorithm 
{
   /**
    * Get the name of the implemented check digit algorithm, Modulus 11.
    * @return algorithm name.
    */
   public String getName() {
      return "Modulus11 Algorithm";
   }
   
   /**
    * Get the symbol of the implemented check digit algorithm. In this
    * case "11" for Modulus 11.
    */  
   public String getSymbol() {
      return "11";
   }

   /**
    * Checks if the supplied sequence matches the Modulus 11 Algorithm.
    * Modulus 11 doesn't use any weights.
    * @param sequence Integer sequence representing the digits to check,
    *                 not necessary between 0 and 9.
    * @param weights  Must be null.
    * @return wether sequences matches the algorithm.
    */
   public boolean check(int[] sequence, int[] weights) {
      long result=0;

      /*
       * Sum up the digits excluding the last digit, the check digit.
       */
      for(int i=0; i<sequence.length-1; i++) 
            result+=sequence[i]*(sequence.length-i);

      result%=11;

      /*
       * This is the special case mentioned above
       */
      if(result==0 && sequence[sequence.length-1]==result)
         return true; 
      
      /*
       * Substract and compare.
       */   
      return 11-(result%11)==sequence[sequence.length-1];
   }
   
   /**
    * The Modulus 11 Algorithm doesn't use weights.
    * @return false
    */
   public boolean usesWeights() {
      return false;
   }

   /**
    * Return the check digit position inside the sequence. Modulus 11 uses
    * the last digit.
    * @param sequence Length of the integer sequence representing the digits
    *                 to check, not necessary between 0 and 9.
    * @return Position of the check digit.
    */
   public int getCheckDigitPosition(int sequence) {
      return sequence-1;
   }
   
}
