/* 
 * THC-Shagg - A modular application to bruteforce check digit algorithms
 * (c) 2001, 2003 Plasmoid <plasmoid@thc.org>, The Hacker's Choice
 * $Id: Modulus9Algorithm.java,v 1.7 2003/07/18 10:08:05 plasmoid Exp $ 
 */
package org.thc.shagg.algorithms;

/**
 * Implementation of the Modulus 7/9 Algorithm. Both algorithms have been
 * used in the hardware implementation of print devices in the early days of
 * the computer. The algorithms are seldomly used due to the fact that they
 * interpret the sequence of digits as number.
 * <p>
 * The Modulus 7/9 Algorithm interpret the supplied sequence of digits as
 * an integer number and the check digit is the result of the Modulus 7
 * respectively the Modulus 9 operation.
 *
 * @version $Id: Modulus9Algorithm.java,v 1.7 2003/07/18 10:08:05 plasmoid Exp $
 * @author Plasmoid (plasmoid@thc.org),
 *                  <a href="http://www.thc.org">THC</a>
 */
public class Modulus9Algorithm implements CheckDigitAlgorithm 
{

   /**
    * Get the name of the implemented check digit algorithm, Modulus 9.
    * @return algorithm name.
    */
   public String getName() {
      return "Modulus9 Algorithm";
   }

   /**
    * Get the symbol of the implemented check digit algorithm. In this
    * case "9" for Modulus 9.
    */
   public String getSymbol() {
      return "9";
   }

   /**
    * Checks if the supplied sequence matches the Modulus 9 Algorithm.
    * Modulus 9 doesn't use any weights.
    * @param sequence Integer sequence representing the digits to check,
    *                 not necessary between 0 and 9.
    * @param weights  Must be null.
    * @return wether sequences matches the algorithm.
    */
   public boolean check(int[] sequence, int[] weights) {
      double result=0;

      /*
       * A natural fact, for any x and d, (x modulus d) < d
       */
      if(sequence[sequence.length-1]>8)
         return false;

      /*
       * Construct the integer number from the single digits. This
       * part may cause an overflow if the double datatype can't hold
       * all digits.
       */
      for(int i=0; i<sequence.length-1; i++)
            result+=sequence[i]*Math.pow(10,sequence.length-i-2);

      return ((int)result%9)==sequence[sequence.length-1];
   }
 
   /**
    * The Modulus 9 Algorithm doesn't use weights.
    * @return false
    */
   public boolean usesWeights() {
      return false;
   }

   /**
    * Return the check digit position inside the sequence. Modulus 9 uses
    * the last digit.
    * @param sequence Length of the integer sequence representing the digits
    *                 to check, not necessary between 0 and 9.
    * @return Position of the check digit.
    */
   public int getCheckDigitPosition(int sequence) {
      return sequence-1;
   }

}
